@@ -21,6 +21,7 @@ from __future__ import division, absolute_import, print_function
 
 import os
 import re
+from platform import python_version
 from collections import namedtuple, Counter
 from itertools import chain
 
@@ -38,6 +39,7 @@ from beets import library
 from beets import config
 from beets import logging
 from beets.util.confit import _package_path
+import six
 
 VARIOUS_ARTISTS = u'Various Artists'
 PromptChoice = namedtuple('ExtraChoice', ['short', 'long', 'callback'])
@@ -85,13 +87,13 @@ def _print_keys(query):
     returned row, with identation of 2 spaces.
     """
     for row in query:
-        print_(' ' * 2 + row['key'])
+        print_(u' ' * 2 + row['key'])
 
 
 def fields_func(lib, opts, args):
     def _print_rows(names):
         names.sort()
-        print_("  " + "\n  ".join(names))
+        print_(u'  ' + u'\n  '.join(names))
 
     print_(u"Item fields:")
     _print_rows(library.Item.all_keys())
@@ -163,7 +165,7 @@ def disambig_string(info):
             else:
                 disambig.append(info.media)
         if info.year:
-            disambig.append(unicode(info.year))
+            disambig.append(six.text_type(info.year))
         if info.country:
             disambig.append(info.country)
         if info.label:
@@ -236,9 +238,9 @@ def show_change(cur_artist, cur_album, match):
             if mediums > 1:
                 return u'{0}-{1}'.format(medium, medium_index)
             else:
-                return unicode(medium_index)
+                return six.text_type(medium_index or index)
         else:
-            return unicode(index)
+            return six.text_type(index)
 
     # Identify the album in question.
     if cur_artist != match.info.artist or \
@@ -279,7 +281,7 @@ def show_change(cur_artist, cur_album, match):
     print_(' '.join(info))
 
     # Tracks.
-    pairs = match.mapping.items()
+    pairs = list(match.mapping.items())
     pairs.sort(key=lambda item_and_track_info: item_and_track_info[1].index)
 
     # Build up LHS and RHS for track difference display. The `lines` list
@@ -754,7 +756,7 @@ class TerminalImportSession(importer.ImportSession):
                     _, _, candidates, rec = autotag.tag_album(
                         task.items, search_ids=search_id.split()
                     )
-            elif choice in extra_ops.keys():
+            elif choice in list(extra_ops.keys()):
                 # Allow extra ops to automatically set the post-choice.
                 post_choice = extra_ops[choice](self, task)
                 if isinstance(post_choice, importer.action):
@@ -771,7 +773,7 @@ class TerminalImportSession(importer.ImportSession):
         either an action constant or a TrackMatch object.
         """
         print_()
-        print_(task.item.path)
+        print_(displayable_path(task.item.path))
         candidates, rec = task.candidates, task.rec
 
         # Take immediate action if appropriate.
@@ -806,7 +808,7 @@ class TerminalImportSession(importer.ImportSession):
                 if search_id:
                     candidates, rec = autotag.tag_item(
                         task.item, search_ids=search_id.split())
-            elif choice in extra_ops.keys():
+            elif choice in list(extra_ops.keys()):
                 # Allow extra ops to automatically set the post-choice.
                 post_choice = extra_ops[choice](self, task)
                 if isinstance(post_choice, importer.action):
@@ -821,8 +823,8 @@ class TerminalImportSession(importer.ImportSession):
         """Decide what to do when a new album or item seems similar to one
         that's already in the library.
         """
-        log.warn(u"This {0} is already in the library!",
-                 (u"album" if task.is_album else u"item"))
+        log.warning(u"This {0} is already in the library!",
+                    (u"album" if task.is_album else u"item"))
 
         if config['import']['quiet']:
             # In quiet mode, don't prompt -- just skip.
@@ -900,9 +902,9 @@ class TerminalImportSession(importer.ImportSession):
                 # Keep the first of the choices, removing the rest.
                 dup_choices = [c for c in all_choices if c.short == short]
                 for c in dup_choices[1:]:
-                    log.warn(u"Prompt choice '{0}' removed due to conflict "
-                             u"with '{1}' (short letter: '{2}')",
-                             c.long, dup_choices[0].long, c.short)
+                    log.warning(u"Prompt choice '{0}' removed due to conflict "
+                                u"with '{1}' (short letter: '{2}')",
+                                c.long, dup_choices[0].long, c.short)
                     extra_choices.remove(c)
         return extra_choices
 
@@ -1044,7 +1046,7 @@ import_cmd.parser.add_option(
 )
 import_cmd.parser.add_option(
     u'-S', u'--search-id', dest='search_ids', action='append',
-    metavar='BACKEND_ID',
+    metavar='ID',
     help=u'restrict matching to a specific metadata backend ID'
 )
 import_cmd.func = import_func
@@ -1053,7 +1055,7 @@ default_commands.append(import_cmd)
 
 # list: Query and show library contents.
 
-def list_items(lib, query, album, fmt=''):
+def list_items(lib, query, album, fmt=u''):
     """Print out items in lib matching query. If album, then search for
     albums instead of single items.
     """
@@ -1079,11 +1081,18 @@ default_commands.append(list_cmd)
 
 # update: Update library contents according to on-disk tags.
 
-def update_items(lib, query, album, move, pretend):
+def update_items(lib, query, album, move, pretend, fields):
     """For all the items matched by the query, update the library to
     reflect the item's embedded tags.
+    :param fields: The fields to be stored. If not specified, all fields will
+    be.
     """
     with lib.transaction():
+        if move and fields is not None and 'path' not in fields:
+            # Special case: if an item needs to be moved, the path field has to
+            # updated; otherwise the new path will not be reflected in the
+            # database.
+            fields.append('path')
         items, _ = _do_query(lib, query, album)
 
         # Walk through the items and pick up their changes.
@@ -1122,24 +1131,25 @@ def update_items(lib, query, album, move, pretend):
                     item._dirty.discard(u'albumartist')
 
             # Check for and display changes.
-            changed = ui.show_model_changes(item,
-                                            fields=library.Item._media_fields)
+            changed = ui.show_model_changes(
+                item,
+                fields=fields or library.Item._media_fields)
 
             # Save changes.
             if not pretend:
                 if changed:
                     # Move the item if it's in the library.
                     if move and lib.directory in ancestry(item.path):
-                        item.move()
+                        item.move(store=False)
 
-                    item.store()
+                    item.store(fields=fields)
                     affected_albums.add(item.album_id)
                 else:
                     # The file's mtime was different, but there were no
                     # changes to the metadata. Store the new mtime,
                     # which is set in the call to read(), so we don't
                     # check this again in the future.
-                    item.store()
+                    item.store(fields=fields)
 
         # Skip album changes while pretending.
         if pretend:
@@ -1158,17 +1168,24 @@ def update_items(lib, query, album, move, pretend):
             # Update album structure to reflect an item in it.
             for key in library.Album.item_keys:
                 album[key] = first_item[key]
-            album.store()
+            album.store(fields=fields)
 
             # Move album art (and any inconsistent items).
             if move and lib.directory in ancestry(first_item.path):
                 log.debug(u'moving album {0}', album_id)
-                album.move()
+
+                # Manually moving and storing the album.
+                items = list(album.items())
+                for item in items:
+                    item.move(store=False)
+                    item.store(fields=fields)
+                album.move(store=False)
+                album.store(fields=fields)
 
 
 def update_func(lib, opts, args):
     update_items(lib, decargs(args), opts.album, ui.should_move(opts.move),
-                 opts.pretend)
+                 opts.pretend, opts.fields)
 
 
 update_cmd = ui.Subcommand(
@@ -1188,37 +1205,43 @@ update_cmd.parser.add_option(
     u'-p', u'--pretend', action='store_true',
     help=u"show all changes but do nothing"
 )
+update_cmd.parser.add_option(
+    u'-F', u'--field', default=None, action='append', dest='fields',
+    help=u'list of fields to update'
+)
 update_cmd.func = update_func
 default_commands.append(update_cmd)
 
 
 # remove: Remove items from library, delete files.
 
-def remove_items(lib, query, album, delete):
+def remove_items(lib, query, album, delete, force):
     """Remove items matching query from lib. If album, then match and
     remove whole albums. If delete, also remove files from disk.
     """
     # Get the matching items.
     items, albums = _do_query(lib, query, album)
 
-    # Prepare confirmation with user.
-    print_()
-    if delete:
-        fmt = u'$path - $title'
-        prompt = u'Really DELETE %i file%s (y/n)?' % \
-                 (len(items), 's' if len(items) > 1 else '')
-    else:
-        fmt = ''
-        prompt = u'Really remove %i item%s from the library (y/n)?' % \
-                 (len(items), 's' if len(items) > 1 else '')
+    # Confirm file removal if not forcing removal.
+    if not force:
+        # Prepare confirmation with user.
+        print_()
+        if delete:
+            fmt = u'$path - $title'
+            prompt = u'Really DELETE %i file%s (y/n)?' % \
+                     (len(items), 's' if len(items) > 1 else '')
+        else:
+            fmt = u''
+            prompt = u'Really remove %i item%s from the library (y/n)?' % \
+                     (len(items), 's' if len(items) > 1 else '')
 
-    # Show all the items.
-    for item in items:
-        ui.print_(format(item, fmt))
+        # Show all the items.
+        for item in items:
+            ui.print_(format(item, fmt))
 
-    # Confirm with user.
-    if not ui.input_yn(prompt, True):
-        return
+        # Confirm with user.
+        if not ui.input_yn(prompt, True):
+            return
 
     # Remove (and possibly delete) items.
     with lib.transaction():
@@ -1227,7 +1250,7 @@ def remove_items(lib, query, album, delete):
 
 
 def remove_func(lib, opts, args):
-    remove_items(lib, decargs(args), opts.album, opts.delete)
+    remove_items(lib, decargs(args), opts.album, opts.delete, opts.force)
 
 
 remove_cmd = ui.Subcommand(
@@ -1237,6 +1260,10 @@ remove_cmd.parser.add_option(
     u"-d", u"--delete", action="store_true",
     help=u"also remove files from disk"
 )
+remove_cmd.parser.add_option(
+    u"-f", u"--force", action="store_true",
+    help=u"do not ask when removing items"
+)
 remove_cmd.parser.add_album_option()
 remove_cmd.func = remove_func
 default_commands.append(remove_cmd)
@@ -1310,6 +1337,7 @@ default_commands.append(stats_cmd)
 
 def show_version(lib, opts, args):
     print_(u'beets version %s' % beets.__version__)
+    print_(u'Python version {}'.format(python_version()))
     # Show plugins.
     names = sorted(p.name for p in plugins.find_plugins())
     if names:
@@ -1601,7 +1629,7 @@ def config_func(lib, opts, args):
             filenames.insert(0, user_path)
 
         for filename in filenames:
-            print_(filename)
+            print_(displayable_path(filename))
 
     # Open in editor.
     elif opts.edit:
@@ -1609,7 +1637,8 @@ def config_func(lib, opts, args):
 
     # Dump configuration.
     else:
-        print_(config.dump(full=opts.defaults, redact=opts.redact))
+        config_out = config.dump(full=opts.defaults, redact=opts.redact)
+        print_(util.text_string(config_out))
 
 
 def config_edit():
@@ -1655,17 +1684,19 @@ default_commands.append(config_cmd)
 
 def print_completion(*args):
     for line in completion_script(default_commands + plugins.commands()):
-        print_(line, end='')
+        print_(line, end=u'')
     if not any(map(os.path.isfile, BASH_COMPLETION_PATHS)):
-        log.warn(u'Warning: Unable to find the bash-completion package. '
-                 u'Command line completion might not work.')
+        log.warning(u'Warning: Unable to find the bash-completion package. '
+                    u'Command line completion might not work.')
 
 BASH_COMPLETION_PATHS = map(syspath, [
     u'/etc/bash_completion',
     u'/usr/share/bash-completion/bash_completion',
-    u'/usr/share/local/bash-completion/bash_completion',
-    u'/opt/local/share/bash-completion/bash_completion',  # SmartOS
-    u'/usr/local/etc/bash_completion',  # Homebrew
+    u'/usr/local/share/bash-completion/bash_completion',
+    # SmartOS
+    u'/opt/local/share/bash-completion/bash_completion',
+    # Homebrew (before bash-completion2)
+    u'/usr/local/etc/bash_completion',
 ])
 
 
@@ -1677,7 +1708,7 @@ def completion_script(commands):
     """
     base_script = os.path.join(_package_path('beets.ui'), 'completion_base.sh')
     with open(base_script, 'r') as base_script:
-        yield base_script.read()
+        yield util.text_string(base_script.read())
 
     options = {}
     aliases = {}
@@ -1692,12 +1723,12 @@ def completion_script(commands):
             if re.match(r'^\w+$', alias):
                 aliases[alias] = name
 
-        options[name] = {'flags': [], 'opts': []}
+        options[name] = {u'flags': [], u'opts': []}
         for opts in cmd.parser._get_all_options()[1:]:
             if opts.action in ('store_true', 'store_false'):
-                option_type = 'flags'
+                option_type = u'flags'
             else:
-                option_type = 'opts'
+                option_type = u'opts'
 
             options[name][option_type].extend(
                 opts._short_opts + opts._long_opts
@@ -1705,14 +1736,14 @@ def completion_script(commands):
 
     # Add global options
     options['_global'] = {
-        'flags': [u'-v', u'--verbose'],
-        'opts': u'-l --library -c --config -d --directory -h --help'.split(
-            u' ')
+        u'flags': [u'-v', u'--verbose'],
+        u'opts':
+            u'-l --library -c --config -d --directory -h --help'.split(u' ')
     }
 
     # Add flags common to all commands
     options['_common'] = {
-        'flags': [u'-h', u'--help']
+        u'flags': [u'-h', u'--help']
     }
 
     # Start generating the script
@@ -1730,14 +1761,17 @@ def completion_script(commands):
 
     # Fields
     yield u"  fields='%s'\n" % ' '.join(
-        set(library.Item._fields.keys() + library.Album._fields.keys())
+        set(
+            list(library.Item._fields.keys()) +
+            list(library.Album._fields.keys())
+        )
     )
 
     # Command options
     for cmd, opts in options.items():
         for option_type, option_list in opts.items():
             if option_list:
-                option_list = ' '.join(option_list)
+                option_list = u' '.join(option_list)
                 yield u"  local %s__%s='%s'\n" % (
                     option_type, cmd, option_list)
 
