@@ -15,10 +15,10 @@
 
 """Determine BPM by pressing a key to the rhythm."""
 
-from __future__ import (division, absolute_import, print_function,
-                        unicode_literals)
+from __future__ import division, absolute_import, print_function
 
 import time
+from six.moves import input
 
 from beets import ui
 from beets.plugins import BeetsPlugin
@@ -32,7 +32,7 @@ def bpm(max_strokes):
     dt = []
     for i in range(max_strokes):
         # Press enter to the rhythm...
-        s = raw_input()
+        s = input()
         if s == '':
             t1 = time.time()
             # Only start measuring at the second stroke
@@ -59,8 +59,8 @@ class BPMPlugin(BeetsPlugin):
 
     def commands(self):
         cmd = ui.Subcommand('bpm',
-                            help='determine bpm of a song by pressing \
-                            a key to the rhythm')
+                            help=u'determine bpm of a song by pressing '
+                            u'a key to the rhythm')
         cmd.func = self.command
         return [cmd]
 
@@ -70,7 +70,7 @@ class BPMPlugin(BeetsPlugin):
     def get_bpm(self, items, write=False):
         overwrite = self.config['overwrite'].get(bool)
         if len(items) > 1:
-            raise ValueError('Can only get bpm of one song at time')
+            raise ValueError(u'Can only get bpm of one song at time')
 
         item = items[0]
         if item['bpm']:
