@@ -29,8 +29,11 @@ from beets import importer
 from beets import ui
 from beets import util
 from beets import config
+from beets.mediafile import image_mime_type
 from beets.util.artresizer import ArtResizer
 from beets.util import confit
+from beets.util import syspath, bytestring_path, py3_path
+import six
 
 try:
     import itunes
@@ -38,9 +41,11 @@ try:
 except ImportError:
     HAVE_ITUNES = False
 
-IMAGE_EXTENSIONS = ['png', 'jpg', 'jpeg']
-CONTENT_TYPES = ('image/jpeg', 'image/png')
-DOWNLOAD_EXTENSION = '.jpg'
+CONTENT_TYPES = {
+    'image/jpeg': [b'jpg', b'jpeg'],
+    'image/png': [b'png']
+}
+IMAGE_EXTENSIONS = [ext for exts in CONTENT_TYPES.values() for ext in exts]
 
 
 class Candidate(object):
@@ -226,30 +231,61 @@ class RemoteArtSource(ArtSource):
         try:
             with closing(self.request(candidate.url, stream=True,
                                       message=u'downloading image')) as resp:
-                if 'Content-Type' not in resp.headers \
-                        or resp.headers['Content-Type'] not in CONTENT_TYPES:
-                    self._log.debug(
-                        u'not a supported image: {}',
-                        resp.headers.get('Content-Type') or u'no content type',
-                    )
-                    candidate.path = None
+                ct = resp.headers.get('Content-Type', None)
+
+                # Download the image to a temporary file. As some servers
+                # (notably fanart.tv) have proven to return wrong Content-Types
+                # when images were uploaded with a bad file extension, do not
+                # rely on it. Instead validate the type using the file magic
+                # and only then determine the extension.
+                data = resp.iter_content(chunk_size=1024)
+                header = b''
+                for chunk in data:
+                    header += chunk
+                    if len(header) >= 32:
+                        # The imghdr module will only read 32 bytes, and our
+                        # own additions in mediafile even less.
+                        break
+                else:
+                    # server didn't return enough data, i.e. corrupt image
                     return
 
-                # Generate a temporary file with the correct extension.
-                with NamedTemporaryFile(suffix=DOWNLOAD_EXTENSION,
-                                        delete=False) as fh:
-                    for chunk in resp.iter_content(chunk_size=1024):
+                real_ct = image_mime_type(header)
+                if real_ct is None:
+                    # detection by file magic failed, fall back to the
+                    # server-supplied Content-Type
+                    # Is our type detection failsafe enough to drop this?
+                    real_ct = ct
+
+                if real_ct not in CONTENT_TYPES:
+                    self._log.debug(u'not a supported image: {}',
+                                    real_ct or u'unknown content type')
+                    return
+
+                ext = b'.' + CONTENT_TYPES[real_ct][0]
+
+                if real_ct != ct:
+                    self._log.warning(u'Server specified {}, but returned a '
+                                      u'{} image. Correcting the extension '
+                                      u'to {}',
+                                      ct, real_ct, ext)
+
+                suffix = py3_path(ext)
+                with NamedTemporaryFile(suffix=suffix, delete=False) as fh:
+                    # write the first already loaded part of the image
+                    fh.write(header)
+                    # download the remaining part of the image
+                    for chunk in data:
                         fh.write(chunk)
                 self._log.debug(u'downloaded art to: {0}',
                                 util.displayable_path(fh.name))
-                candidate.path = fh.name
+                candidate.path = util.bytestring_path(fh.name)
                 return
 
         except (IOError, requests.RequestException, TypeError) as exc:
             # Handling TypeError works around a urllib3 bug:
             # https://github.com/shazow/urllib3/issues/556
             self._log.debug(u'error fetching art: {}', exc)
-            candidate.path = None
             return
 
 
@@ -566,7 +602,7 @@ class Wikipedia(RemoteArtSource):
         try:
             data = wikipedia_response.json()
             results = data['query']['pages']
-            for _, result in results.iteritems():
+            for _, result in results.items():
                 image_url = result['imageinfo'][0]['url']
                 yield self._candidate(url=image_url,
                                       match=Candidate.MATCH_EXACT)
@@ -594,20 +630,22 @@ class FileSystem(LocalArtSource):
         paths = extra['paths']
         if not paths:
             return
-        cover_names = extra['cover_names']
-        cover_pat = br"(\b|_)({0})(\b|_)".format(b'|'.join(cover_names))
+        cover_names = list(map(util.bytestring_path, extra['cover_names']))
+        cover_names_str = b'|'.join(cover_names)
+        cover_pat = br''.join([br"(\b|_)(", cover_names_str, br")(\b|_)"])
         cautious = extra['cautious']
 
         for path in paths:
-            if not os.path.isdir(path):
+            if not os.path.isdir(syspath(path)):
                 continue
 
             # Find all files that look like images in the directory.
             images = []
-            for fn in os.listdir(path):
+            for fn in os.listdir(syspath(path)):
+                fn = bytestring_path(fn)
                 for ext in IMAGE_EXTENSIONS:
-                    if fn.lower().endswith(b'.' + ext.encode('utf8')) and \
-                       os.path.isfile(os.path.join(path, fn)):
+                    if fn.lower().endswith(b'.' + ext) and \
+                       os.path.isfile(syspath(os.path.join(path, fn))):
                         images.append(fn)
 
             # Look for "preferred" filenames.
@@ -691,7 +729,7 @@ class FetchArtPlugin(plugins.BeetsPlugin, RequestMixin):
                           confit.String(pattern=self.PAT_PERCENT)]))
         self.margin_px = None
         self.margin_percent = None
-        if type(self.enforce_ratio) is unicode:
+        if type(self.enforce_ratio) is six.text_type:
             if self.enforce_ratio[-1] == u'%':
                 self.margin_percent = float(self.enforce_ratio[:-1]) / 100
             elif self.enforce_ratio[-2:] == u'px':
@@ -702,7 +740,7 @@ class FetchArtPlugin(plugins.BeetsPlugin, RequestMixin):
             self.enforce_ratio = True
 
         cover_names = self.config['cover_names'].as_str_seq()
-        self.cover_names = map(util.bytestring_path, cover_names)
+        self.cover_names = list(map(util.bytestring_path, cover_names))
         self.cautious = self.config['cautious'].get(bool)
         self.store_source = self.config['store_source'].get(bool)
 
@@ -725,7 +763,8 @@ class FetchArtPlugin(plugins.BeetsPlugin, RequestMixin):
         if 'remote_priority' in self.config:
             self._log.warning(
                 u'The `fetch_art.remote_priority` configuration option has '
-                u'been deprecated, see the documentation.')
+                u'been deprecated. Instead, place `filesystem` at the end of '
+                u'your `sources` list.')
             if self.config['remote_priority'].get(bool):
                 try:
                     sources_name.remove(u'filesystem')
