@@ -24,9 +24,9 @@ import json
 import re
 import requests
 import unicodedata
-import urllib
 import warnings
-from HTMLParser import HTMLParseError
+import six
+from six.moves import urllib
 
 try:
     from bs4 import SoupStrainer, BeautifulSoup
@@ -40,6 +40,15 @@ try:
 except ImportError:
     HAS_LANGDETECT = False
 
+try:
+    # PY3: HTMLParseError was removed in 3.5 as strict mode
+    # was deprecated in 3.3.
+    # https://docs.python.org/3.3/library/html.parser.html
+    from six.moves.html_parser import HTMLParseError
+except ImportError:
+    class HTMLParseError(Exception):
+        pass
+
 from beets import plugins
 from beets import ui
 
@@ -70,12 +79,12 @@ URL_CHARACTERS = {
 def unescape(text):
     """Resolve &#xxx; HTML entities (and some others)."""
     if isinstance(text, bytes):
-        text = text.decode('utf8', 'ignore')
+        text = text.decode('utf-8', 'ignore')
     out = text.replace(u'&nbsp;', u' ')
 
     def replchar(m):
         num = m.group(1)
-        return unichr(int(num))
+        return six.unichr(int(num))
     out = re.sub(u"&#(\d+);", replchar, out)
     return out
 
@@ -135,29 +144,32 @@ def search_pairs(item):
     The method also tries to split multiple titles separated with `/`.
     """
 
+    def generate_alternatives(string, patterns):
+        """Generate string alternatives by extracting first matching group for
+           each given pattern."""
+        alternatives = [string]
+        for pattern in patterns:
+            match = re.search(pattern, string, re.IGNORECASE)
+            if match:
+                alternatives.append(match.group(1))
+        return alternatives
+
     title, artist = item.title, item.artist
-    titles = [title]
-    artists = [artist]
 
-    # Remove any featuring artists from the artists name
-    pattern = r"(.*?) {0}".format(plugins.feat_tokens())
-    match = re.search(pattern, artist, re.IGNORECASE)
-    if match:
-        artists.append(match.group(1))
+    patterns = [
+        # Remove any featuring artists from the artists name
+        r"(.*?) {0}".format(plugins.feat_tokens())]
+    artists = generate_alternatives(artist, patterns)
 
-    # Remove a parenthesized suffix from a title string. Common
-    # examples include (live), (remix), and (acoustic).
-    pattern = r"(.+?)\s+[(].*[)]$"
-    match = re.search(pattern, title, re.IGNORECASE)
-    if match:
-        titles.append(match.group(1))
-
-    # Remove any featuring artists from the title
-    pattern = r"(.*?) {0}".format(plugins.feat_tokens(for_artist=False))
-    for title in titles[:]:
-        match = re.search(pattern, title, re.IGNORECASE)
-        if match:
-            titles.append(match.group(1))
+    patterns = [
+        # Remove a parenthesized suffix from a title string. Common
+        # examples include (live), (remix), and (acoustic).
+        r"(.+?)\s+[(].*[)]$",
+        # Remove any featuring artists from the title
+        r"(.*?) {0}".format(plugins.feat_tokens(for_artist=False)),
+        # Remove part of title after colon ':' for songs with subtitles
+        r"(.+?)\s*:.*"]
+    titles = generate_alternatives(title, patterns)
 
     # Check for a dual song (e.g. Pink Floyd - Speak to Me / Breathe)
     # and each of them.
@@ -177,11 +189,11 @@ class Backend(object):
     @staticmethod
     def _encode(s):
         """Encode the string for inclusion in a URL"""
-        if isinstance(s, unicode):
+        if isinstance(s, six.text_type):
             for char, repl in URL_CHARACTERS.items():
                 s = s.replace(char, repl)
-            s = s.encode('utf8', 'ignore')
-        return urllib.quote(s)
+            s = s.encode('utf-8', 'ignore')
+        return urllib.parse.quote(s)
 
     def build_url(self, artist, title):
         return self.URL_PATTERN % (self._encode(artist.title()),
@@ -218,12 +230,12 @@ class SymbolsReplaced(Backend):
         '>': 'Greater_Than',
         '#': 'Number_',
         r'[\[\{]': '(',
-        r'[\[\{]': ')'
+        r'[\]\}]': ')',
     }
 
     @classmethod
     def _encode(cls, s):
-        for old, new in cls.REPLACEMENTS.iteritems():
+        for old, new in cls.REPLACEMENTS.items():
             s = re.sub(old, new, s)
 
         return super(SymbolsReplaced, cls)._encode(s)
@@ -250,13 +262,13 @@ class Genius(Backend):
     """Fetch lyrics from Genius via genius-api."""
     def __init__(self, config, log):
         super(Genius, self).__init__(config, log)
-        self.api_key = config['genius_api_key'].get(unicode)
+        self.api_key = config['genius_api_key'].as_str()
         self.headers = {'Authorization': "Bearer %s" % self.api_key}
 
     def search_genius(self, artist, title):
         query = u"%s %s" % (artist, title)
         url = u'https://api.genius.com/search?q=%s' \
-            % (urllib.quote(query.encode('utf8')))
+            % (urllib.parse.quote(query.encode('utf-8')))
 
         self._log.debug(u'genius: requesting search {}', url)
         try:
@@ -461,8 +473,8 @@ class Google(Backend):
     """Fetch lyrics from Google search results."""
     def __init__(self, config, log):
         super(Google, self).__init__(config, log)
-        self.api_key = config['google_API_key'].get(unicode)
-        self.engine_id = config['google_engine_ID'].get(unicode)
+        self.api_key = config['google_API_key'].as_str()
+        self.engine_id = config['google_engine_ID'].as_str()
 
     def is_lyrics(self, text, artist=None):
         """Determine whether the text seems to be valid lyrics.
@@ -503,7 +515,7 @@ class Google(Backend):
         try:
             text = unicodedata.normalize('NFKD', text).encode('ascii',
                                                               'ignore')
-            text = unicode(re.sub('[-\s]+', ' ', text.decode('utf-8')))
+            text = six.text_type(re.sub('[-\s]+', ' ', text.decode('utf-8')))
         except UnicodeDecodeError:
             self._log.exception(u"Failing to normalize '{0}'", text)
         return text
@@ -542,9 +554,9 @@ class Google(Backend):
         query = u"%s %s" % (artist, title)
         url = u'https://www.googleapis.com/customsearch/v1?key=%s&cx=%s&q=%s' \
               % (self.api_key, self.engine_id,
-                 urllib.quote(query.encode('utf8')))
+                 urllib.parse.quote(query.encode('utf-8')))
 
-        data = urllib.urlopen(url)
+        data = urllib.request.urlopen(url)
         data = json.load(data)
         if 'error' in data:
             reason = data['error']['errors'][0]['reason']
@@ -615,9 +627,9 @@ class LyricsPlugin(plugins.BeetsPlugin):
                                 u'no API key configured.')
                 sources.remove('google')
             elif not HAS_BEAUTIFUL_SOUP:
-                self._log.warn(u'To use the google lyrics source, you must '
-                               u'install the beautifulsoup4 module. See the '
-                               u'documentation for further details.')
+                self._log.warning(u'To use the google lyrics source, you must '
+                                  u'install the beautifulsoup4 module. See '
+                                  u'the documentation for further details.')
                 sources.remove('google')
 
         self.config['bing_lang_from'] = [
@@ -625,9 +637,9 @@ class LyricsPlugin(plugins.BeetsPlugin):
         self.bing_auth_token = None
 
         if not HAS_LANGDETECT and self.config['bing_client_secret'].get():
-            self._log.warn(u'To use bing translations, you need to '
-                           u'install the langdetect module. See the '
-                           u'documentation for further details.')
+            self._log.warning(u'To use bing translations, you need to '
+                              u'install the langdetect module. See the '
+                              u'documentation for further details.')
 
         self.backends = [self.SOURCE_BACKENDS[source](self.config, self._log)
                          for source in sources]
@@ -643,7 +655,7 @@ class LyricsPlugin(plugins.BeetsPlugin):
         oauth_url = 'https://datamarket.accesscontrol.windows.net/v2/OAuth2-13'
         oauth_token = json.loads(requests.post(
             oauth_url,
-            data=urllib.urlencode(params)).content)
+            data=urllib.parse.urlencode(params)).content)
         if 'access_token' in oauth_token:
             return "Bearer " + oauth_token['access_token']
         else:
@@ -754,7 +766,7 @@ class LyricsPlugin(plugins.BeetsPlugin):
                     self.bing_auth_token = None
                     return self.append_translation(text, to_lang)
                 return text
-            lines_translated = ET.fromstring(r.text.encode('utf8')).text
+            lines_translated = ET.fromstring(r.text.encode('utf-8')).text
             # Use a translation mapping dict to build resulting lyrics
             translations = dict(zip(text_lines, lines_translated.split('|')))
             result = ''
